/*
 *This file is part of the Configuration File Library (CFL).
 * Copyright (C) 2002 Nicolas Darnis <ndarnis@free.fr>
 *
 * The CFL library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * The CFL library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the CFL library; see the file COPYING.
 * If not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 * File: parser.y
 * $Revision: 1.4 $
 * $Date: 2002/11/06 11:45:23 $
 * $Author: ndarnis $
 * $Id: parser.y,v 1.4 2002/11/06 11:45:23 ndarnis Exp $
 */

%{
#include <config.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <gdsl.h>

#include "cfl.h"
#include "parser.h"
#include "scanner.h"

static char *section_name = NULL;
static cflh_t handle = NULL;
static int rc = 0;

static int yy_error (const char *msg);
%}

%union 
{
  bool *    boolean;
  char *    string;
  char *    identifier;
  long int *integer;
  double *  real;
}

%token <boolean>    BOOLEAN
%token <string>     STRING
%token <integer>    INTEGER
%token <real>       REAL
%token <identifier> IDENTIFIER

%token AFFECT OPEN CLOSE

%start begin

%%
begin:         sections
               ;

sections:      /* EPSILON */
               | section sections 
               ;

section:       OPEN IDENTIFIER CLOSE 
               { 
		 if ($2 == NULL) YYABORT;
		 if (section_name != NULL) {free (section_name); section_name = NULL;}
                 section_name = $2;
		 if (cfl_section_insert (handle, $2) < 0) YYABORT;
               }
               section_body
               ;

section_body: /* EPSILON */
               | section_line section_body
               ;

section_line: IDENTIFIER AFFECT STRING 
               { 
		 if ($1 == NULL || $3 == NULL) YYABORT;
                 rc = cfl_value_insert (handle, section_name, $1, CFL_TYPE_STRING, $3); 
		 free ($1);
		 free ($3);
                 if (rc < 0) YYABORT;
               }
               | IDENTIFIER AFFECT INTEGER 
               {
		 if ($1 == NULL || $3 == NULL) YYABORT;
                 rc = cfl_value_insert (handle, section_name, $1, CFL_TYPE_INTEGER, $3);
		 free ($1);
		 free ($3);
                 if (rc < 0) YYABORT;
               }
               | IDENTIFIER AFFECT REAL 
               { 
		 if ($1 == NULL || $3 == NULL) YYABORT;
                 rc = cfl_value_insert (handle, section_name, $1, CFL_TYPE_FLOAT, $3);
		 free ($1);
		 free ($3);
                 if (rc < 0) YYABORT;
               }
               | IDENTIFIER AFFECT BOOLEAN
               {
                 if ($1 == NULL || $3 == NULL) YYABORT;
                 rc = cfl_value_insert (handle, section_name, $1, CFL_TYPE_BOOL, (void *) $3);
                 free ($1);
		 free ($3);
                 if (rc < 0) YYABORT;
               }
               ;
%%

/******************************************************************************/
/* Public functions                                                           */
/******************************************************************************/

extern int
yy_parse_file (const FILE *input_file, cflh_t h)
{
  int rc;
  handle = h;

  yy_set_input_file (input_file);

  rc = yy_parse ();

  if (section_name != NULL)
    {
      free (section_name);
      section_name = NULL;
    }

  return (rc == 0) ? 0 : -1;
}

/******************************************************************************/
/* Private functions                                                          */
/******************************************************************************/

static int yy_error (const char *s)
{
  return 1;
}
